﻿using System;
using Eneter.Messaging.EndPoints.TypedMessages;
using Eneter.Messaging.MessagingSystems.MessagingSystemBase;
using Eneter.Messaging.MessagingSystems.TcpMessagingSystem;

namespace HelloWorldTcpService
{
    class Program
    {
        static void Main(string[] args)
        {
            // Create message receiver.
            // It receives string and responses also string.
            IDuplexTypedMessagesFactory aTypedMessagesFactory = new DuplexTypedMessagesFactory();
            IDuplexTypedMessageReceiver<string, string> aMessageReceiver
                = aTypedMessagesFactory.CreateDuplexTypedMessageReceiver<string, string>();

            // Subscribe to receive messages.
            aMessageReceiver.MessageReceived += OnMessageReceived;

            // Create input channel that will listen messages via TCP.
            IMessagingSystemFactory aTcpMessaging = new TcpMessagingSystemFactory();
            IDuplexInputChannel anInputChannel = aTcpMessaging.CreateDuplexInputChannel("tcp://127.0.0.1:8055/");

            // Attach the input channel to the message receiver and start listening.
            aMessageReceiver.AttachDuplexInputChannel(anInputChannel);

            Console.WriteLine("TCP service is running. To stop press ENTER.");
            Console.ReadLine();

            // Detach the input channel and stop listening.
            aMessageReceiver.DetachDuplexInputChannel();
        }


        private static void OnMessageReceived(object sender, TypedRequestReceivedEventArgs<string> e)
        {
            // Display receive message.
            Console.WriteLine("Received message: " + e.RequestMessage);

            // Send back simple response.
            string aResponseMessage = "Thanks for " + e.RequestMessage;
            IDuplexTypedMessageReceiver<string, string> aReceiver
                = (IDuplexTypedMessageReceiver<string, string>)sender;
            aReceiver.SendResponseMessage(e.ResponseReceiverId, aResponseMessage);
        }
    }
}
